local addonName, SourceLord = ...
_G.SourceLord = SourceLord

SourceLord.data = {}

---- Options

local fEvents = CreateFrame("Frame", "SourceLord_EventsFrame")

local backdrop = {
	--bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
	bgFile="Interface\\ChatFrame\\ChatFrameBackground", 
	edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", 
	tile=1, tileSize=10, edgeSize=16, 
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}

local BonusTooltip = {}
do
	local baseName = "SourceLordTooltip_"

	local function fixOffsets(self, offset)
		local prefix = self:GetName() .. "TextRight"
		
		for lineNum = 1, self:NumLines() do
			local fs = _G[prefix .. lineNum]

			if not fs then
				return
			elseif fs:IsVisible() then
				local numPoints = fs:GetNumPoints()
				for n=1, numPoints do
					local point, rel, relpoint, x, y = fs:GetPoint(n)
					--D(prefix .. lineNum, n, point, rel, relpoint, x, y)
					if x then
						fs:SetPoint(point, rel, relpoint, x + offset, y)
					end
				end
			end

		end
	end

	local function fixSize (self)
		local parent = self.parent

		local tw = self:GetWidth()
		local pw = parent:GetWidth()
		local defecit = pw - tw
		if defecit > 0.05 then
			self:SetWidth(pw)
			fixOffsets(self, defecit)
		elseif defecit < -0.05 then
			parent:SetWidth(tw)
			fixOffsets(parent, -defecit)
		end

		self:SetScript("OnUpdate", nil)
	end

	local function AttachTo (self, parent)
		self.parent = parent
		self:SetParent(parent)
		self:SetOwner(parent, "ANCHOR_NONE")
		--self:SetPoint("TOP",tooltip,"BOTTOM")

		--self:SetPoint("TOP", tooltip, "BOTTOM", 0, 3)
		self:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", 0, 3)
		--self:SetPoint("TOPRIGHT", tooltip, "BOTTOMRIGHT", 0, 3)

		self:SetBackdrop(backdrop)
		self:SetScale(1)
		self:SetBackdropColor(0, 0, 0, 0.9)
		self:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)

		_G[self:GetName() .. "TextLeft1"]:SetFontObject(GameFontNormal)	
	end

	local function QueueAutoResize (self)
		self:SetScript("OnUpdate", fixSize)
	end

	function BonusTooltip:Create (name)
		--D("Creating", baseName .. name)
		local tip = CreateFrame("GameTooltip", baseName .. name, UIParent, "GameTooltipTemplate")
		-- tip:SetFrameStrata("TOOLTIP")
		-- tip:SetFrameLevel(100)

		tip.AttachTo = AttachTo
		tip.QueueAutoResize = QueueAutoResize

		return tip
	end
end

local sideMap = {
	["Alliance"] = 1,
	["Horde"] = 2,

	[1] = "Alliance",
	[2] = "Horde",
}
local classMap = {
	["WARRIOR"] = 1,
	["PALADIN"] = 2,
	["HUNTER"] = 4,
	["ROGUE"] = 8,
	["PRIEST"] = 16,
	["DEATHKNIGHT"] = 32,
	["SHAMAN"] = 64,
	["MAGE"] = 128,
	["WARLOCK"] = 256,
	["MONK"] = 512,
	["DRUID"] = 1024,
}
local subclassMap = {
	["Cloth"] = 1,
	["Leather"] = 2,
	["Mail"] = 3,
	["Plate"] = 4,

	["Daggers"] = 15,
	["Fist Weapons"] = 13,
	["One-Handed Axes"] = 0,
	["One-Handed Maces"] = 4,
	["One-Handed Swords"] = 7,
	["Polearms"] = 6,
	["Staves"] = 10,
	["Two-Handed Axes"] = 1,
	["Two-Handed Maces"] = 5,
	["Two-Handed Swords"] = 8,
	["Bows"] = 2,
	["Crossbows"] = 18,
	["Guns"] = 3,
	["Thrown"] = 16,
	["Wands"] = 19,
	["Fishing Poles"] = 20,
	["Miscellaneous"] = 14,
}

local colors = {
  vendor = "ffff00",
  blackmarket = "806000",
  pvp = "ff8000",
  crafted = "ffffb0",  
  other = "00d0d0",
  other_low = "00d0d0",  
  worldboss = "008080",
  drop = "40ff40",
  dropzone = "00a000", 
  dropspec = "a0ffa0",
  dungeon = "0070dd",
  dungeonh = "0050bd",  
  raid = "a335ee",
  raidh = "8315ce",
  raidm = "53009e",  
  quest = "ff0000",
  unavail = "800000",
  unknown = "ff60a0",

	_world = "ffffff",

	cannotuse = "404040",
	alliance = "2060ff",
	horde = "ff2020",
	append = "777777",
}

local function annotateItem (item, itemID, itemVariant, parentID, parentSubclass, playerSide, playerClassBit)
	local name = item[1]
	local displayid = item[2]
	local subclass = item[3]
	local reqclass = item[4]
	local side = item[5]
	local unavail = item[6]
	local sourcetag = item[7]

	local tag = sourcetag
	local append = ""

	-- http://www.wowhead.com/item=114440?xml
  -- 1=heroic, 4=???(challengemode?) 5=levelvariable, 6/7/8=stage1/2/3

	if (itemVariant) then
		append = ("%s [Variant %s]"):format(append, itemVariant)
	end

	if (parentSubclass ~= nil) and (subclass >= 0) then
	-- think we can ignore negative subclasses, only matters for 4 armor types (+weapons?)
		if (parentSubclass ~= subclass) then
			tag = "cannotuse"
			local rev = "Unknown type"
			for k,v in pairs(subclassMap) do
				if v == subclass then
					rev = k
					break
				end
			end
			append = ("%s (%s)"):format(append, rev)
		end
	end

	if (reqclass ~= 0) then
		if bit.band(reqclass, playerClassBit) == 0 then
			tag = "cannotuse"
			append = ("%s (%s)"):format(append, "Class Restricted")
		else
			append = ("%s (%s)"):format(append, "Class OK")
		end
	end

	if (side ~= 0) then
		if (side ~= playerSide) then
			tag = "cannotuse"
		end

		if side == 1 then
			append = ("%s (%s)"):format(append, sideMap[side])
		else
			append = ("%s (%s)"):format(append, sideMap[side])
		end
	end

	if (unavail ~= 0) then
		tag = "unavail"
	end	

	--D(playerSide, playerClassBit)
	return ("|cff%s%s|cff%s%s|r"):format(colors[tag], name, colors["append"], append)
end

function showSourceTooltip (parent, itemID, src, dm)
	local tip = parent.sourceTooltip
	if not tip then
		tip = BonusTooltip:Create(parent:GetName())
		parent.sourceTooltip = tip
	end

	local parentName = parent:GetName()
	if (parentName ~= "ItemRefTooltip") and (not IsControlKeyDown()) then
		return
	end

	local itemName, _, _, _, _, itemType, itemSubType = GetItemInfo(itemID)
	if (itemName == nil) then
		-- D("NIL!", itemID)
		return
	end

	tip:AttachTo(parent)

	local str = ""
	if src then
		if (src[6] == 1) then
			tip:AddLine("- |cff"..colors["unavail"].."No longer available|r to players", 0.65, 0.65, 0.65)
		elseif (src[6] == 2) then
			tip:AddLine("- |cff"..colors["unavail"].."Not available|r to players", 0.65, 0.65, 0.65)
		end

		local side = src[5]
		if side == 1 then
			tip:AddLine("- |cff" .. colors["alliance"] .. "Alliance|r only", 0.65, 0.65, 0.65)
		elseif side == 2 then
			tip:AddLine("- |cff" .. colors["horde"] .. "Horde|r only", 0.65, 0.65, 0.65)
		end

		if src[8] then
			for i = 8, #src do
				tip:AddLine("- " .. src[i], 0.65, 0.65, 0.65)
			end
		else
			tip:AddLine("- From an |cff"..colors["unknown"].."unknown source|r", 0.65, 0.65, 0.65)
		end

		showTip = true
	end

	if dm then
		local _order = {}
		local _map = {}
		for i = 1, #dm do
			local entry = dm[i]
			local entryID = nil
			local entryVariant = nil
			if (type(entry) == "table") then
				entryID = entry[1]
				entryVariant = entry[2]
			else
				entryID = entry
			end

			if (entryVariant) then
				if (_map[entryID]) then
					_map[entryID] = _map[entryID] .. "/" .. tostring(entryVariant)
				else
					_map[entryID] = tostring(entryVariant)
					tinsert(_order,entryID)
				end
			else
				tinsert(_order,entryID)
			end
		end

		dm = {}
		for i = 1, #_order do
			local entryID = _order[i]
			if (_map[entryID]) then
				tinsert(dm, { entryID, _map[entryID] })
			else
				tinsert(dm, entryID)
			end
		end

		local f = UnitFactionGroup("player")
		local playerSide = sideMap[f]

		local _, cn = UnitClass("player")
		local playerClassBit = classMap[cn]

		local parentSubclass = subclassMap[itemSubType]

		tip:AddLine("- Shares an appearance with:", 0.65, 0.65, 0.65)
		for i = 1, #dm do
			local entry = dm[i]
			local entryID = -1
			local entryVariant = nil

			if (type(entry) == "table") then
				entryID = entry[1]
				entryVariant = entry[2]
			else
				entryID = entry
			end

			if (entryID ~= itemID) then
				local item = SourceLord.data.itemData[entryID]
				if item then
					local displayName = annotateItem(item, entryID, entryVariant, itemID, parentSubclass, playerSide, playerClassBit)
					tip:AddLine("    " .. displayName, 0.65, 0.65, 0.65)
				end
			end
		end

		showTip = true
	else
		--tip:AddLine("- Unique appearance", 0.65, 0.65, 0.65)
	end

	tip:Show()
	tip:QueueAutoResize()
end

local function showSource (self, ...)
	local itemLink = select(2, self:GetItem())
	if not itemLink then
		return
	end

	local _, _, itemID = string.find(itemLink, "|Hitem:(%d+):")
	itemID = tonumber(itemID)

	local src = SourceLord.data.itemData[itemID]
	local dm = nil
	if (src) and (src[2] ~= 0) then
		dm = SourceLord.data.displayMap[src[2]]
	end

	if (src ~= nil) then
		showSourceTooltip(self, itemID, src, dm)
	elseif (self.sourceTooltip) then
		self.sourceTooltip:Hide()
	end
end

local function HookTip(tooltip)
	tooltip:HookScript('OnTooltipSetItem', showSource)
end

fEvents:SetScript("OnEvent", function(self, event, name)
	if name ~= addonName then return end
	
	self:UnregisterEvent("ADDON_LOADED")
	self:SetScript("OnEvent", SourceLord_HandleEvent)
	
	HookTip(GameTooltip)
	HookTip(ItemRefTooltip)
	if AtlasLootTooltip then
		HookTip(AtlasLootTooltip)
	end

end)

fEvents:RegisterEvent("ADDON_LOADED")
